/* ************************************************************************

   Copyright:

   License:

   Authors:

************************************************************************ */

qx.Theme.define("callbackery.theme.Color", {
    extend : qx.theme.indigo.Color,
    colors : {
        "tabview-page-background": "#fdfdfd",
        "tabview-page-border": "#eee",
        "tabview-button-background": "white",
        "tabview-button-checked-background": "tabview-page-background",
        "tabview-button-text": "black",
        "tabview-button-checked-text": "black",
        "tabview-button-border": "#fdfdfd",
        "tabview-button-checked-border": "tabview-page-border",
        "textfield-readonly": "#eee"
    }
});

